@@ -2,12 +2,22 @@ from typing import Tuple
 import torch
 import torch.nn.functional as F
 
+from pyiqa.utils.color_util import to_y_channel
 from pyiqa.matlab_utils import fspecial, imfilter
 from .arch_util import excact_padding_2d
 
 EPS = torch.finfo(torch.float32).eps
 
 
+def preprocess_rgb(x, test_y_channel, data_range=1., color_space='yiq'):
+    if test_y_channel and x.shape[1] == 3:
+        x = to_y_channel(x, data_range, color_space)
+    else:
+        x = x * data_range
+        x = x - x.detach() + x.round()
+    return x
+
+
 def extract_2d_patches(x, kernel, stride=1, dilation=1, padding='same'):
     """
     Ref: https://stackoverflow.com/a/65886666
@@ -43,6 +53,12 @@ def safe_sqrt(x: torch.Tensor) -> torch.Tensor:
     return torch.sqrt(x + EPS)
 
 
+def diff_round(x: torch.Tensor) -> torch.Tensor:
+    r"""Differentiable round.
+    """
+    return x - x.detach() + x.round()
+
+
 def normalize_img_with_guass(img: torch.Tensor,
                              kernel_size: int = 7,
                              sigma: float = 7. / 6,
