@@ -19,9 +19,10 @@ from pyiqa.utils.color_util import to_y_channel
 from pyiqa.utils.download_util import load_file_from_url
 from pyiqa.matlab_utils import imresize, fspecial, SCFpyr_PyTorch, dct2d, im2col
 from pyiqa.archs.func_util import extract_2d_patches
-from pyiqa.archs.ssim_arch import SSIM
+from pyiqa.archs.ssim_arch import ssim as ssim_func
 from pyiqa.archs.arch_util import ExactPadding2d
 from pyiqa.archs.niqe_arch import NIQE
+from warnings import warn
 
 default_model_urls = {'url': 'https://github.com/chaofengc/IQA-PyTorch/releases/download/v0.1-weights/NRQM_model.mat'}
 
@@ -49,8 +50,11 @@ def get_var_gen_gauss(x, eps=1e-7):
     return rho
 
 
-def gamma_gen_gauss(x: Tensor):
+def gamma_gen_gauss(x: Tensor, block_seg=1e4):
     r"""General gaussian distribution estimation.
+
+    Args:
+        block_seg: maximum number of blocks in parallel to avoid OOM
     """
     pshape = x.shape[:-1]
     x = x.reshape(-1, x.shape[-1])
@@ -65,7 +69,16 @@ def gamma_gen_gauss(x: Tensor):
 
     rho = var / (mean_abs + eps)
 
-    indexes = (rho - r_table).abs().argmin(dim=-1)
+    if rho.shape[0] > block_seg:
+        rho_seg = rho.chunk(int(rho.shape[0] // block_seg))
+        indexes = []
+        for r in rho_seg:
+            tmp_idx = (r - r_table).abs().argmin(dim=-1)
+            indexes.append(tmp_idx)
+        indexes = torch.cat(indexes)
+    else:
+        indexes = (rho - r_table).abs().argmin(dim=-1)
+
     solution = gamma[indexes].reshape(*pshape)
     return solution
 
@@ -161,7 +174,7 @@ def block_dct(img: Tensor):
     return dct_feat
 
 
-def norm_sender_normalized(pyr, num_scale=2, num_bands=6, blksz=3):
+def norm_sender_normalized(pyr, num_scale=2, num_bands=6, blksz=3, eps=1e-12):
     r"""Normalize pyramid with local spatial neighbor and band neighbor
     """
     border = blksz // 2
@@ -198,7 +211,7 @@ def norm_sender_normalized(pyr, num_scale=2, num_bands=6, blksz=3):
             L, Q = torch.linalg.eigh(C_x)
             L_pos = L * (L > 0)
             L_pos_sum = L_pos.sum(dim=1, keepdim=True)
-            L = L_pos * L.sum(dim=1, keepdim=True) / (L_pos_sum + (L_pos_sum == 0).float())
+            L = L_pos * L.sum(dim=1, keepdim=True) / (L_pos_sum + (L_pos_sum == 0).to(L.dtype))
             C_x = Q @ torch.diag_embed(L) @ Q.transpose(1, 2)
 
             o_c = current_band[:, border:-border, border:-border]
@@ -206,10 +219,20 @@ def norm_sender_normalized(pyr, num_scale=2, num_bands=6, blksz=3):
             o_c = o_c.reshape(b, hw)
             o_c = o_c - o_c.mean(dim=1, keepdim=True)
 
-            tmp_y = (tmp @ torch.linalg.pinv(C_x)) * tmp / N
+            if tmp.shape[1] >= 2e5: # To avoid out of GPU memory
+                C_x = C_x.cpu()
+                tmp = tmp.cpu()
+            if hasattr(torch.linalg, 'lstsq'):
+                tmp_y = torch.linalg.lstsq(C_x.transpose(1, 2), tmp.transpose(1, 2)).solution.transpose(1, 2) * tmp / N
+            else:
+                warn(
+                    "For numerical stability, we use torch.linal.lstsq to calculate matrix inverse for PyTorch > 1.9.0. The results might be slightly different if you use older version of PyTorch.")
+                tmp_y = (tmp @ torch.linalg.pinv(C_x)) * tmp / N
+            tmp_y = tmp_y.to(o_c)
+
             z = tmp_y.sum(dim=2).sqrt()
             mask = z != 0
-            g_c = o_c.masked_select(mask) / z.masked_select(mask)
+            g_c = o_c * mask / (z * mask + eps)
             g_c = g_c.reshape(b, h, w)
 
             gb = int(guardband / (2**(si)))
@@ -244,16 +267,16 @@ def global_gsm(img: Tensor):
 
     # structure correlation between scales
     hp_band = pyr[0]
-    ssim_func = SSIM(channels=1, test_y_channel=False)
-    for sb in subbands:
-        sb_tmp = imresize(sb, sizes=hp_band.shape[1:]).unsqueeze(1)
-        tmp_ssim = ssim_func(sb_tmp, hp_band.unsqueeze(1))
-        feat.append(tmp_ssim)
+    for sb in lp_bands:
+        curr_band = imresize(sb, sizes=hp_band.shape[1:]).unsqueeze(1)
+        _, tmpscore = ssim_func(curr_band, hp_band.unsqueeze(1), get_cs=True, data_range=255)
+        feat.append(tmpscore)
 
     # structure correlation between orientations
     for i in range(num_bands):
         for j in range(i + 1, num_bands):
-            feat.append(ssim_func(subbands[i].unsqueeze(1), subbands[j].unsqueeze(1)))
+            _, tmpscore = ssim_func(subbands[i].unsqueeze(1), subbands[j].unsqueeze(1), get_cs=True, data_range=255)
+            feat.append(tmpscore)
 
     feat = torch.stack(feat, dim=1)
     return feat
@@ -293,7 +316,7 @@ def random_forest_regression(feat, ldau, rdau, threshold_value, pred_value, best
                                          best_attri[:, i])
             tmp_pred.append(tmp_result)
         pred.append(tmp_pred)
-    pred = torch.Tensor(pred)
+    pred = torch.tensor(pred)
     return pred.mean(dim=1, keepdim=True)
 
 
@@ -312,7 +335,8 @@ def nrqm(
 
     # crop image
     b, c, h, w = img.shape
-    img_pyr = get_guass_pyramid(img.float() / 255.)
+    img = img.double()
+    img_pyr = get_guass_pyramid(img / 255.)
 
     # DCT features
     f1 = []
@@ -327,7 +351,7 @@ def nrqm(
     f3 = []
     for im in img_pyr:
         col = im2col(im, 5, 'distinct')
-        _, s, _ = torch.linalg.svd(col)
+        _, s, _ = torch.linalg.svd(col, full_matrices=False)
         f3.append(s)
     f3 = torch.cat(f3, dim=1)
 
@@ -336,7 +360,7 @@ def nrqm(
     for feat, rf in zip([f1, f2, f3], rf_param):
         tmp_pred = random_forest_regression(feat, *rf)
         preds = torch.cat((preds, tmp_pred), dim=1)
-    quality = preds @ torch.Tensor(linear_param)
+    quality = preds @ torch.tensor(linear_param)
 
     return quality.squeeze()
 
@@ -373,7 +397,7 @@ def calculate_nrqm(img: torch.Tensor,
 
     if test_y_channel and img.shape[1] == 3:
         img = to_y_channel(img, 255, color_space)
-
+    
     if crop_border != 0:
         img = img[..., crop_border:-crop_border, crop_border:-crop_border]
 
